Module: PuppetBox::Util
- Defined in:
- lib/puppetbox/util.rb
Class Method Summary collapse
-
.test_file_name(node_name, test_name) ⇒ Object
return the filename that a testcase for a particular test should live in eg node_name=centos7, test_name=role::base should be in centos7/role__base.pp.
Class Method Details
.test_file_name(node_name, test_name) ⇒ Object
return the filename that a testcase for a particular test should live in eg node_name=centos7, test_name=role::base should be in centos7/role__base.pp
7 8 9 10 |
# File 'lib/puppetbox/util.rb', line 7 def self.test_file_name(node_name, test_name) test_name_safe = test_name.gsub(/::/,'__') File.join(node_name, "#{test_name_safe}.pp") end |